/*
 * Copyright (C) 2013-2017 Fairphone Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.fairphone.fplauncher3.edgeswipe.editor;

import java.util.ArrayList;

import com.fairphone.fplauncher3.R;

import android.content.ComponentName;
import android.content.Context;
import android.content.SharedPreferences;

/**
 * Favorites storage helper class implements methods to convert a string array
 * into one string so it can be stored easily in the shared preferences. This is
 * used to store the selected apps for the favorite app launcher.
 */
public abstract class FavoritesStorageHelper
{
    private static final String TAG = FavoritesStorageHelper.class.getSimpleName();

    private static final String PACKAGE_AND_CLASS_NAME_DELIMITER = ",";
    private static final String COMPONENT_NAME_DELIMITER = ";";
    private static final String FAVORITES_APPS_SHARED_PREFERENCES_FILE_NAME = "FAVORITES_APPS_SHARED_PREFERENCES_FILE_NAME";
    private static final String FAVORITES_APPS_KEY = "FAVORITES_APPS_KEY";
    private static final int MAX_FAVORITE_APPS = 4;


    /**
     * Load the favorites apps from the shared preferences and get the component
     * names from each.
     */
    public static ArrayList<ComponentName> getComponentNamesOfFavorites(
            Context context) {

        String componentNamesString = getComponentNamesString(context);

        if (componentNamesString != null) {
            ArrayList<ComponentName> favorites = stringToComponentNameArray(componentNamesString);

            if (!hasOnlyNullEntries(favorites)) {
                return favorites;
            }

            /* Only null references were stored in edgeswipe menu. Probably
             * user had removed all icons. Try getting old default apps.
             */
            String defaultApps = getDefaultEdgeSwipeApps(context);
            if (defaultApps != null) {
                return stringToComponentNameArray(defaultApps);
            }
        }
        return new ArrayList<ComponentName>();
    }

    /**
     * Get component names from preferences.
     *
     * Fall back to legacy default component names in case that string is null.
     */
    private static String getComponentNamesString(Context context) {
        String componentNamesString = getSharedPrefs(context).getString(
                FAVORITES_APPS_KEY, null);

        if (componentNamesString == null || componentNamesString.isEmpty()) {
            componentNamesString = getDefaultEdgeSwipeApps(context);
        }

        return componentNamesString;
    }

    private static boolean hasOnlyNullEntries(ArrayList<ComponentName> list) {
        for (ComponentName element : list) {
            if (element != null) {
                return false;
            }
        }
        return true;
    }

    public static boolean hasFavoritesString(Context context) {
        return getSharedPrefs(context).contains(FAVORITES_APPS_KEY);
    }

    public static void clearLegacySettings(Context context) {
        getSharedPrefs(context).edit().clear().apply();
    }

    private static SharedPreferences getSharedPrefs(Context context) {
        return context.getSharedPreferences(FAVORITES_APPS_SHARED_PREFERENCES_FILE_NAME,
                Context.MODE_PRIVATE);
    }

    public static String getDefaultEdgeSwipeApps(Context context) {
        return context.getResources().getString(R.string.edge_swipe_default_apps);
    }

    /**
     * Convert a string into an array of strings, using the delimiter as
     * separator.
     */
    public static String[] stringToArray(String input, String delimiter){
        return input.split(delimiter);
    }

    public static ArrayList<ComponentName> stringToComponentNameArray(String input) {
        String[] componentNamesStringArray = FavoritesStorageHelper.stringToArray(input.replaceAll("\\s",""), COMPONENT_NAME_DELIMITER);
        ArrayList<ComponentName> componentNamesArray = new ArrayList<ComponentName>();
        for (int i=0; i< componentNamesStringArray.length; i++) {
            if(!componentNamesStringArray[i].equals(PACKAGE_AND_CLASS_NAME_DELIMITER)){
                ComponentName compName = FavoritesStorageHelper.stringToComponentName(componentNamesStringArray[i]);
                componentNamesArray.add(compName);
            }else{
                componentNamesArray.add(null);
            }
        }
        return componentNamesArray;
    }

    public static ComponentName stringToComponentName(String componentNameString)
    {
        ComponentName compName = null;
        String[] componentNameArray = FavoritesStorageHelper.stringToArray(componentNameString, PACKAGE_AND_CLASS_NAME_DELIMITER);
        if (componentNameArray.length == 2)
        {
            compName = new ComponentName(componentNameArray[0], componentNameArray[1]);
        }
        return compName;
    }
}
