package com.fairphone.fplauncher3.helper;

import android.app.usage.UsageStats;
import android.app.usage.UsageStatsManager;
import android.content.Context;
import android.os.Build;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.Map;

import com.android.launcher3.compat.LauncherActivityInfoCompat;
import com.android.launcher3.compat.LauncherAppsCompat;
import com.android.launcher3.compat.UserHandleCompat;
import com.android.launcher3.util.ComponentKey;

/**
 * Generates list of most used apps from {@link android.app.usage.UsageStats}.
 *
 * Returns data for primary user. Does not return usage data for all users in
 * {@link com.android.launcher3.compat.UserManagerCompatVL.getUserProfiles()}.
 * since we do not get separate usage data for profiles and could not
 * distinguish which icon to show.
 */
public class UsageStatsHelper {

    // Constant Context.USAGE_STATS_SERVICE is only introduced in API 22.
    private static final String USAGE_STATS_SERVICE = "usagestats";

    /**
     * Indicates whether usage data is available.
     *
     * {@link android.app.usage.UsageStats} has been introduced in API level 21.
     */
    public static boolean isUsageDataAvailable() {
        return Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP;
    }

    /**
     * Return {@link ComponentKey} for most most used apps in descending order
     * (subject to availability).
     */
    public static List<ComponentKey> getMostUsedApps(Context context) {
        Map<String, UsageStats> usageStats = getUsageStats(context);
        List<String> packageNames = toSortedPackageNames(usageStats);

        return toComponentKeys(context, packageNames);
    }

    /**
     * Sort usageStats by most time in foreground and extract package names.
     */
    private static List<String> toSortedPackageNames(Map<String, UsageStats> usageStatsMap) {
        List<UsageStats> usageStatsList = new ArrayList<>(usageStatsMap.values());
        Collections.sort(usageStatsList, new MostUsedComparator());

        List<String> packageNames = new ArrayList<>(usageStatsList.size());

        for (UsageStats us : usageStatsList) {
            if (us != null) {
                packageNames.add(us.getPackageName());
            }
        }
        return packageNames;
    }

    /**
     * Get {@link ComponentKey} of first enabled activity for each package name.
     *
     * Only considers current user.
     */
    private static List<ComponentKey> toComponentKeys(Context context, List<String> packageNames) {
        LauncherAppsCompat launcherApps = LauncherAppsCompat.getInstance(context);
        UserHandleCompat user = UserHandleCompat.myUserHandle();

        List<ComponentKey> componentKeys = new ArrayList<>(packageNames.size());
        for (String packageName : packageNames) {
            ComponentKey key = getComponentKeyOfFirstActivity(launcherApps, packageName, user);
            if (key != null) {
                componentKeys.add(key);
            }
        }
        return componentKeys;
    }

    /**
     * Find the first enabled launcher activity for {@code packageName} and
     * {@code user} and return {@link ComponentKey} for it.
     *
     * Returns {@code null} if no such activity exists.
     */
    private static ComponentKey getComponentKeyOfFirstActivity(LauncherAppsCompat launcherApps,
            String packageName, UserHandleCompat user) {
        List<LauncherActivityInfoCompat> activities = launcherApps.getActivityList(packageName, user);

        for (LauncherActivityInfoCompat activity : activities) {
            if (launcherApps.isActivityEnabledForProfile(activity.getComponentName(), user)) {
                return new ComponentKey(activity.getComponentName(), activity.getUser());
            }
        }
        return null;
    }

    /**
     * Get usage stats for previous 7 days.
     */
    private static Map<String, UsageStats> getUsageStats(Context context) {
        Calendar cal = Calendar.getInstance();
        cal.add(Calendar.DAY_OF_MONTH, -7);
        final long from = cal.getTimeInMillis();
        final long to = System.currentTimeMillis();

        UsageStatsManager usm = (UsageStatsManager) context.getSystemService(USAGE_STATS_SERVICE);
        return usm.queryAndAggregateUsageStats(from, to);
    }

    /**
     * Implementation of {@link Comparator} that compares {@code left} and
     * {@code right} based on total time in foreground.
     */
    private static class MostUsedComparator implements Comparator<UsageStats> {
        @Override
        public int compare(UsageStats left, UsageStats right) {
            if (left == right) {
                return 0;
            }
            if (left == null) {
                return -1;
            }
            if (right == null) {
                return 1;
            }
            return Long.compare(right.getTotalTimeInForeground(), left.getTotalTimeInForeground());
        }
    }
}
